/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ivory.workflow;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.ivory.IvoryException;
import org.apache.ivory.entity.v0.Entity;
import org.apache.ivory.util.ReflectionUtils;

public abstract class WorkflowBuilder<T extends Entity> {

    public static WorkflowBuilder<Entity> getBuilder(String engine, Entity entity) throws IvoryException {
        String classKey = engine + "." + entity.getEntityType().name().toLowerCase() + ".workflow.builder";
        return ReflectionUtils.getInstance(classKey);
    }

    public abstract Map<String, Properties> newWorkflowSchedule(T entity, List<String> clusters) throws IvoryException;
    
    public abstract Properties newWorkflowSchedule(T entity, Date startDate, String clusterName, String user) throws IvoryException;

    public abstract String[] getWorkflowNames(T entity);
}